function [vipgh vipga gaussh gaussa info allcen iters] = hubauthgauss(A,nnodes,opts,show)
%HUBAUTHGAUSS Hub/authority centrality computation via Gauss quadrature.
%   [nodeh nodea] = HUBAUTHGAUSS(A) identifies the most important nodes,
%   according to hub and authority centrality, of a network whose adjacency
%   matrix is A.
%
%   [nodesh nodesa] = HUBAUTHGAUSS(A,nnodes,opts,show) adds more information.
% 	nnodes: number of nodes to be identified (default 1).
%	opts: additional options, see code.
%	show: if set to 1, displays pictures and results (default 0).
%
%   [nodesh nodesa valsh valsa info] = HUBAUTHGAUSS(A,...) gets more info.
%	nodesh, nodesa: list of nodes.
%	valsh, valsa: approximation of hub/auth centrality for each node.
%	info: info(1)=0 means success, info(1)=1 means that a convergence 
%	      test was not satisfied (see code). info(2) contains the number of
%	      matrix vector products performed during the computation.

%   James Baglama, University of Rhode Island, U.S.A.
%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   Lothar Reichel, Kent State University, U.S.A.
%   Email: jbaglama@math.uri.edu, reichel@math.kent.edu
%          kate.fenu@gmail.com, rodriguez@unica.it, 
%
%   Last revised Ago 4, 2013

if nargin < 4 || isempty(show),  show = 0;                        end
if nargin < 3 || isempty(opts),  opts = struct('empty','empty');  end
if nargin < 2, error('Too few input arguments.');		  end

nam = fieldnames(opts);
% tolerance for quadrature convergence
if ~any(strcmpi('gausstolq',nam)),  opts.gausstolq = 1e-3; end
% max number of iterations
if ~any(strcmpi('gaussmaxn',nam)),  opts.gaussmaxn = 200;  end

n = size(A,1);

allcen = zeros(2*n,1);
iters = zeros(n,1);
ei = zeros(n,1);
mvp = 0;
info = 0;
%Afun = @(x) func(A,x);
%Afun = @(x) [A*x(n+1:2*n);A'*x(1:n)];

% for gausscosh
%opts.gaussmu = sqrt(eigs(A'*A,1,'LM'));
opts.gaussmu = svds(A,1,'L');
theta = opts.gaussmu;
%opts.gaussreorth = 1;
%opts.gaussmgs = 1;
%opts.gausstheta = norm(A,inf);

for i = 1:n
	%if mod(i,50)==0, i, end
	ei(i) = 1;
	[allcen(i) infog] = gausscosh(A,ei,theta,opts,show);
	info = info || infog(1);
	iters(i) = infog(2);
	mvp = mvp + infog(2);
	[allcen(i+n) infog] = gausscosh(A',ei,theta,opts,show);
	info = info || infog(1);
	iters(i+n) = infog(2);
	mvp = mvp + infog(2);
	ei(i) = 0;
	%[i iters(i)]
end

[gaussh vipgh] = sort(allcen(1:n),'descend');
[gaussa vipga] = sort(allcen(n+1:2*n),'descend');

vipgh = vipgh(1:nnodes);
gaussh = gaussh(1:nnodes);
vipga = vipga(1:nnodes);
gaussa = gaussa(1:nnodes);

if any(iters) >= opts.gaussmaxn
	info = 1;
end
info = [info mvp];


function x = func(B,v)

m = size(B,1);
x = [B*(v(m+1:2*m));B'*(v(1:m))];

